# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet import (
    SimpleResp,
    AssetListResponse,
    AssetCreateResponse,
    AssetRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAsset:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.create(
            key="key=API_KEY",
        )
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.create(
            key="key=API_KEY",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            custom_id="custom_id",
            description="description",
            meta_data={},
            name="name",
            tags=["string"],
        )
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.create(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.create(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(AssetCreateResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.retrieve(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.update(
            id="id",
            key="key=API_KEY",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            description="description",
            meta_data={},
            name="name",
            tags=["string"],
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.list(
            key="key=API_KEY",
        )
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.list(
            key="key=API_KEY",
            cluster="america",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            pn=0,
            ps=100,
            sort="updated_at:desc",
            tags="tags=tag_1,tag_2",
        )
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(AssetListResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.delete(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_bind(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_bind(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_bind(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_bind(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.bind(
                id="",
                key="key=API_KEY",
                device_id="device_id",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_track(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_track_with_all_params(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
                "accuracy": 0,
                "altitude": 0,
                "battery_level": 0,
                "bearing": 0,
                "meta_data": '{\n  "driver_name": "Tyler Durden",\n  "type": "parcel"\n}',
                "speed": 0,
                "tracking_mode": "tracking_mode",
            },
            cluster="america",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_track(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_track(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_track(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.track(
                id="",
                key="key=API_KEY",
                device_id="device_id",
                locations={
                    "location": {
                        "lat": 0,
                        "lon": 0,
                    },
                    "timestamp": 0,
                },
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_attributes(self, client: NextbillionSDK) -> None:
        asset = client.skynet.asset.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_attributes(self, client: NextbillionSDK) -> None:
        response = client.skynet.asset.with_raw_response.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_attributes(self, client: NextbillionSDK) -> None:
        with client.skynet.asset.with_streaming_response.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_attributes(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.asset.with_raw_response.update_attributes(
                id="",
                key="key=API_KEY",
                attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            )


class TestAsyncAsset:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.create(
            key="key=API_KEY",
        )
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.create(
            key="key=API_KEY",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            custom_id="custom_id",
            description="description",
            meta_data={},
            name="name",
            tags=["string"],
        )
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.create(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(AssetCreateResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.create(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(AssetCreateResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.retrieve(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(AssetRetrieveResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.update(
            id="id",
            key="key=API_KEY",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            description="description",
            meta_data={},
            name="name",
            tags=["string"],
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.list(
            key="key=API_KEY",
        )
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.list(
            key="key=API_KEY",
            cluster="america",
            include_all_of_attributes="include_all_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            include_any_of_attributes="include_any_of_attributes=vehicle_type:pickup_truck|driver_name:John",
            pn=0,
            ps=100,
            sort="updated_at:desc",
            tags="tags=tag_1,tag_2",
        )
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(AssetListResponse, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(AssetListResponse, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.delete(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_bind(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_bind(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_bind(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.bind(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_bind(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.bind(
                id="",
                key="key=API_KEY",
                device_id="device_id",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_track(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_track_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
                "accuracy": 0,
                "altitude": 0,
                "battery_level": 0,
                "bearing": 0,
                "meta_data": '{\n  "driver_name": "Tyler Durden",\n  "type": "parcel"\n}',
                "speed": 0,
                "tracking_mode": "tracking_mode",
            },
            cluster="america",
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_track(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_track(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.track(
            id="id",
            key="key=API_KEY",
            device_id="device_id",
            locations={
                "location": {
                    "lat": 0,
                    "lon": 0,
                },
                "timestamp": 0,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_track(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.track(
                id="",
                key="key=API_KEY",
                device_id="device_id",
                locations={
                    "location": {
                        "lat": 0,
                        "lon": 0,
                    },
                    "timestamp": 0,
                },
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_attributes(self, async_client: AsyncNextbillionSDK) -> None:
        asset = await async_client.skynet.asset.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        )
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_attributes(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.asset.with_raw_response.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        asset = await response.parse()
        assert_matches_type(SimpleResp, asset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_attributes(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.asset.with_streaming_response.update_attributes(
            id="id",
            key="key=API_KEY",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            asset = await response.parse()
            assert_matches_type(SimpleResp, asset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_attributes(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.asset.with_raw_response.update_attributes(
                id="",
                key="key=API_KEY",
                attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["MonitorCreateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Unique ID of the `monitor` created. Please note this ID cannot be updated."""


class MonitorCreateResponse(BaseModel):
    data: Optional[Data] = None
    """A data object containing the ID of the `monitor` created."""

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import SnapToRoadSnapResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSnapToRoads:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_snap(self, client: NextbillionSDK) -> None:
        snap_to_road = client.snap_to_roads.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        )
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_snap_with_all_params(self, client: NextbillionSDK) -> None:
        snap_to_road = client.snap_to_roads.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
            approaches="`unrestricted`",
            avoid="toll",
            geometry="`polyline`",
            mode="`car`",
            option="`flexible`",
            radiuses="radiuses=14|16|14",
            road_info="`max_speed`",
            timestamps="timestamps=1656570000|1656570015|1656570030",
            tolerate_outlier=True,
        )
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_snap(self, client: NextbillionSDK) -> None:
        response = client.snap_to_roads.with_raw_response.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        snap_to_road = response.parse()
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_snap(self, client: NextbillionSDK) -> None:
        with client.snap_to_roads.with_streaming_response.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            snap_to_road = response.parse()
            assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSnapToRoads:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_snap(self, async_client: AsyncNextbillionSDK) -> None:
        snap_to_road = await async_client.snap_to_roads.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        )
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_snap_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        snap_to_road = await async_client.snap_to_roads.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
            approaches="`unrestricted`",
            avoid="toll",
            geometry="`polyline`",
            mode="`car`",
            option="`flexible`",
            radiuses="radiuses=14|16|14",
            road_info="`max_speed`",
            timestamps="timestamps=1656570000|1656570015|1656570030",
            tolerate_outlier=True,
        )
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_snap(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.snap_to_roads.with_raw_response.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        snap_to_road = await response.parse()
        assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_snap(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.snap_to_roads.with_streaming_response.snap(
            key="key=API_KEY",
            path="path=41.38602272,2.17621539|41.38312885,2.17207083|41.38157854,2.17906668|41.38288511,2.18186215",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            snap_to_road = await response.parse()
            assert_matches_type(SnapToRoadSnapResponse, snap_to_road, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import DirectionComputeRouteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDirections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_compute_route(self, client: NextbillionSDK) -> None:
        direction = client.directions.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        )
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_compute_route_with_all_params(self, client: NextbillionSDK) -> None:
        direction = client.directions.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
            altcount=1,
            alternatives=True,
            approaches="unrestricted;;curb;",
            avoid="toll",
            bearings="0,180;0,180",
            cross_border=True,
            departure_time=0,
            drive_time_limits="500,400,400",
            emission_class="euro0",
            exclude="toll",
            geometry="polyline",
            hazmat_type="general",
            mode="car",
            option="fast",
            overview="full",
            rest_times="500,300,100",
            road_info="max_speed",
            route_type="fastest",
            steps=True,
            truck_axle_load=0,
            truck_size="200,210,600",
            truck_weight=1,
            turn_angle_range=0,
            waypoints="41.349302,2.136480|41.349303,2.136481|41.349304,2.136482",
        )
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_compute_route(self, client: NextbillionSDK) -> None:
        response = client.directions.with_raw_response.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        direction = response.parse()
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_compute_route(self, client: NextbillionSDK) -> None:
        with client.directions.with_streaming_response.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            direction = response.parse()
            assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDirections:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_compute_route(self, async_client: AsyncNextbillionSDK) -> None:
        direction = await async_client.directions.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        )
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_compute_route_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        direction = await async_client.directions.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
            altcount=1,
            alternatives=True,
            approaches="unrestricted;;curb;",
            avoid="toll",
            bearings="0,180;0,180",
            cross_border=True,
            departure_time=0,
            drive_time_limits="500,400,400",
            emission_class="euro0",
            exclude="toll",
            geometry="polyline",
            hazmat_type="general",
            mode="car",
            option="fast",
            overview="full",
            rest_times="500,300,100",
            road_info="max_speed",
            route_type="fastest",
            steps=True,
            truck_axle_load=0,
            truck_size="200,210,600",
            truck_weight=1,
            turn_angle_range=0,
            waypoints="41.349302,2.136480|41.349303,2.136481|41.349304,2.136482",
        )
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_compute_route(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.directions.with_raw_response.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        direction = await response.parse()
        assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_compute_route(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.directions.with_streaming_response.compute_route(
            destination="41.349302,2.136480",
            origin="41.349302,2.136480",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            direction = await response.parse()
            assert_matches_type(DirectionComputeRouteResponse, direction, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .monitor import Monitor
from ..._models import BaseModel
from .pagination import Pagination

__all__ = ["MonitorListResponse", "Data"]


class Data(BaseModel):
    list: Optional[List[Monitor]] = None
    """An array of objects listing all the monitors.

    Each object represents one `monitor`.
    """

    page: Optional[Pagination] = None
    """An object with pagination details of the search results.

    Use this object to implement pagination in your application.
    """


class MonitorListResponse(BaseModel):
    data: Optional[Data] = None
    """A data object containing the result."""

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypeAlias

__all__ = ["MetadataParam"]

MetadataParam: TypeAlias = object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .place import (
    PlaceResource,
    AsyncPlaceResource,
    PlaceResourceWithRawResponse,
    AsyncPlaceResourceWithRawResponse,
    PlaceResourceWithStreamingResponse,
    AsyncPlaceResourceWithStreamingResponse,
)
from .multigeocode import (
    MultigeocodeResource,
    AsyncMultigeocodeResource,
    MultigeocodeResourceWithRawResponse,
    AsyncMultigeocodeResourceWithRawResponse,
    MultigeocodeResourceWithStreamingResponse,
    AsyncMultigeocodeResourceWithStreamingResponse,
)

__all__ = [
    "PlaceResource",
    "AsyncPlaceResource",
    "PlaceResourceWithRawResponse",
    "AsyncPlaceResourceWithRawResponse",
    "PlaceResourceWithStreamingResponse",
    "AsyncPlaceResourceWithStreamingResponse",
    "MultigeocodeResource",
    "AsyncMultigeocodeResource",
    "MultigeocodeResourceWithRawResponse",
    "AsyncMultigeocodeResourceWithRawResponse",
    "MultigeocodeResourceWithStreamingResponse",
    "AsyncMultigeocodeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TripUpdateParams", "Stop"]


class TripUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    asset_id: Required[str]
    """Use this param to update the ID of the asset which made this trip.

    Please be cautious when using this field as providing an ID other than what was
    provided at the time of starting the trip, will link a new asset to the trip and
    un-link the original asset, even if the trip is still active.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    attributes: object
    """Use this field to update the `attributes` of the trip.

    Please note that when updating the `attributes` field, previously added
    information will be overwritten.
    """

    description: str
    """Use this parameter to update the custom description of the trip."""

    meta_data: object
    """Use this JSON object to update additional details about the trip.

    This property is used to add any custom information / context about the trip.

    Please note that updating the `meta_data` field will overwrite the previously
    added information.
    """

    name: str
    """Use this property to update the name of the trip."""

    stops: Iterable[Stop]
    """Use this object to update the details of the stops made during the trip.

    Each object represents a single stop.

    Please note that when updating this field, the new stops will overwrite any
    existing stops configured for the trip.
    """


class Stop(TypedDict, total=False):
    geofence_id: str
    """
    Use this parameter to update the ID of the geofence indicating the area where
    the asset needs to make a stop, during the trip. Only the IDs of geofences
    created using
    [NextBillion.ai's Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence#create-a-geofence)
    are accepted.

    Please note that updating this field will overwrite the previously added
    information.
    """

    meta_data: object
    """Use this JSON object to update additional details about the stop.

    This property is used to add any custom information / context about the stop.

    Please note that updating the `meta_data` field will overwrite the previously
    added information.
    """

    name: str
    """Use this filed to update the name of the stop."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .document_template_content_response import DocumentTemplateContentResponse

__all__ = ["DocumentTemplateListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the document template."""

    content: Optional[List[DocumentTemplateContentResponse]] = None
    """
    An array of objects returning the details of data structures and validation
    rules and other properties of all document fields. Each object represents one
    document field.
    """

    name: Optional[str] = None
    """Returns the name of the document template."""


class DocumentTemplateListResponse(BaseModel):
    data: Optional[List[Data]] = None
    """
    An array of objects returning the details of each document template associated
    with the specified API key. Each object represents one document template. In
    case there are no templates associated with the given key, a blank array is
    returned.
    """

    msg: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the HTTP response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .step_create_params import StepCreateParams as StepCreateParams
from .step_delete_params import StepDeleteParams as StepDeleteParams
from .step_update_params import StepUpdateParams as StepUpdateParams
from .route_steps_response import RouteStepsResponse as RouteStepsResponse
from .step_complete_params import StepCompleteParams as StepCompleteParams
from .step_create_response import StepCreateResponse as StepCreateResponse
from .step_delete_response import StepDeleteResponse as StepDeleteResponse
from .step_update_response import StepUpdateResponse as StepUpdateResponse
from .document_submission_param import DocumentSubmissionParam as DocumentSubmissionParam
from .route_steps_request_param import RouteStepsRequestParam as RouteStepsRequestParam
from .route_step_completion_mode import RouteStepCompletionMode as RouteStepCompletionMode
from .route_step_geofence_config import RouteStepGeofenceConfig as RouteStepGeofenceConfig
from .route_step_geofence_config_param import RouteStepGeofenceConfigParam as RouteStepGeofenceConfigParam



from __future__ import annotations

import pytest

from nextbillion_sdk._utils import required_args


def test_too_many_positional_params() -> None:
    @required_args(["a"])
    def foo(a: str | None = None) -> str | None:
        return a

    with pytest.raises(TypeError, match=r"foo\(\) takes 1 argument\(s\) but 2 were given"):
        foo("a", "b")  # type: ignore


def test_positional_param() -> None:
    @required_args(["a"])
    def foo(a: str | None = None) -> str | None:
        return a

    assert foo("a") == "a"
    assert foo(None) is None
    assert foo(a="b") == "b"

    with pytest.raises(TypeError, match="Missing required argument: 'a'"):
        foo()


def test_keyword_only_param() -> None:
    @required_args(["a"])
    def foo(*, a: str | None = None) -> str | None:
        return a

    assert foo(a="a") == "a"
    assert foo(a=None) is None
    assert foo(a="b") == "b"

    with pytest.raises(TypeError, match="Missing required argument: 'a'"):
        foo()


def test_multiple_params() -> None:
    @required_args(["a", "b", "c"])
    def foo(a: str = "", *, b: str = "", c: str = "") -> str | None:
        return f"{a} {b} {c}"

    assert foo(a="a", b="b", c="c") == "a b c"

    error_message = r"Missing required arguments.*"

    with pytest.raises(TypeError, match=error_message):
        foo()

    with pytest.raises(TypeError, match=error_message):
        foo(a="a")

    with pytest.raises(TypeError, match=error_message):
        foo(b="b")

    with pytest.raises(TypeError, match=error_message):
        foo(c="c")

    with pytest.raises(TypeError, match=r"Missing required argument: 'a'"):
        foo(b="a", c="c")

    with pytest.raises(TypeError, match=r"Missing required argument: 'b'"):
        foo("a", c="c")


def test_multiple_variants() -> None:
    @required_args(["a"], ["b"])
    def foo(*, a: str | None = None, b: str | None = None) -> str | None:
        return a if a is not None else b

    assert foo(a="foo") == "foo"
    assert foo(b="bar") == "bar"
    assert foo(a=None) is None
    assert foo(b=None) is None

    # TODO: this error message could probably be improved
    with pytest.raises(
        TypeError,
        match=r"Missing required arguments; Expected either \('a'\) or \('b'\) arguments to be given",
    ):
        foo()


def test_multiple_params_multiple_variants() -> None:
    @required_args(["a", "b"], ["c"])
    def foo(*, a: str | None = None, b: str | None = None, c: str | None = None) -> str | None:
        if a is not None:
            return a
        if b is not None:
            return b
        return c

    error_message = r"Missing required arguments; Expected either \('a' and 'b'\) or \('c'\) arguments to be given"

    with pytest.raises(TypeError, match=error_message):
        foo(a="foo")

    with pytest.raises(TypeError, match=error_message):
        foo(b="bar")

    with pytest.raises(TypeError, match=error_message):
        foo()

    assert foo(a=None, b="bar") == "bar"
    assert foo(c=None) is None
    assert foo(c="foo") == "foo"



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SnapToRoadSnapParams"]


class SnapToRoadSnapParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    path: Required[str]
    """
    Pipe-separated list of coordinate points along a path which would be snapped to
    a road.
    """

    approaches: Literal["`unrestricted`", "`curb`"]
    """
    A semicolon-separated list indicating the side of the road from which to
    approach the locations on the snapped route. When set to "unrestricted" a route
    can arrive at the snapped location from either side of the road and when set to
    "curb" the route will arrive at the snapped location on the driving side of the
    region. Please note the number of values provided must be equal to the number of
    coordinate points provided in the "path" parameter. However, you can skip a
    coordinate and show its position in the list with the ";" separator.
    """

    avoid: Literal["toll", "ferry", "highway", "none"]
    """Setting this will ensure the route avoids ferries, tolls, highways or nothing.

    Multiple values should be separated by a pipe (|). If "none" is provided along
    with other values, an error is returned as a valid route is not feasible. Please
    note that when this parameter is not provided in the input, ferries are set to
    be avoided by default. When this parameter is provided, only the mentioned
    objects are avoided.
    """

    geometry: Literal["`polyline`", "`polyline6`", "`geojson`"]
    """Sets the output format of the route geometry in the response.

    Only the "polyline" or "polyline6" encoded "geometry" of the snapped path is
    returned in the response depending on the value provided in the input. When
    "geojson" is selected as the input value, "polyline6" encoded geometry of the
    snapped path is returned along with a "geojson" object.
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine a route.

    For example, if you use "car", the API will return a route that a car can take.
    Using "truck" will return a route a truck can use, taking into account
    appropriate truck routing restrictions.

    Note: Only the "car" profile is enabled by default. Please note that customized
    profiles (including "truck") might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    option: Literal["`flexible`"]
    """
    Include this parameter in the request to return segment-wise speed information
    of the route returned in the response.

    Please note that returning speed information is a function of "road_info"
    parameter, which is effective only when "option=flexible". However, the
    resultant route might not contain all the locations provided in "path" input.
    """

    radiuses: str
    """
    Pipe separated radiuses, in meters (m), up to which a coordinate point can be
    snapped. Please note, if no valid road is available within the specified radius,
    the API would snap the points to nearest, most viable road. When using this
    parameter, it is recommended to specify as many radius values as the number of
    points in "path" parameter. If the same number of "radiuses" are not provided,
    the API will use the default radius value of 25 meters for all locations.
    """

    road_info: Literal["`max_speed`"]
    """
    Use this parameter to receive segment-wise maximum speed information of the
    route in the response. "max_speed" is the only allowed value.
    """

    timestamps: str
    """Pipe-separated UNIX epoch timestamp in seconds for each location.

    If used, the number of timestamps must be equal to the number of coordinate
    points in the "path" parameter. The "timestamps" must increase monotonically
    starting from the first timestamp. This means that each subsequent timestamp
    should either be more than or equal to the preceding one.
    """

    tolerate_outlier: bool
    """Enable it to ignore locations outside the service boundary.

    When "true", the service would ignore "path" coordinates points falling outside
    the accessible area, which otherwise would cause an error when this parameter is
    "false".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import (
    PostResponse,
    OptimizationComputeResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOptimization:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_compute(self, client: NextbillionSDK) -> None:
        optimization = client.optimization.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        )
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_compute_with_all_params(self, client: NextbillionSDK) -> None:
        optimization = client.optimization.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
            approaches="`unrestricted`",
            destination="`any`",
            geometries="`polyline`",
            mode="`car`",
            roundtrip=True,
            source="`any`",
            with_geometry=True,
        )
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_compute(self, client: NextbillionSDK) -> None:
        response = client.optimization.with_raw_response.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        optimization = response.parse()
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_compute(self, client: NextbillionSDK) -> None:
        with client.optimization.with_streaming_response.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            optimization = response.parse()
            assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_re_optimize(self, client: NextbillionSDK) -> None:
        optimization = client.optimization.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        )
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_re_optimize_with_all_params(self, client: NextbillionSDK) -> None:
        optimization = client.optimization.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
            job_changes={
                "add": [
                    {
                        "id": '"id":"Job 1"',
                        "location_index": 0,
                        "delivery": [0],
                        "depot_ids": ["string"],
                        "description": "description",
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "outsourcing_cost": 0,
                        "pickup": [0],
                        "priority": 0,
                        "revenue": 0,
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "skills": [1],
                        "time_windows": [[0]],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "modify": [
                    {
                        "id": '"id":"Job 1"',
                        "location_index": 0,
                        "delivery": [0],
                        "depot_ids": ["string"],
                        "description": "description",
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "outsourcing_cost": 0,
                        "pickup": [0],
                        "priority": 0,
                        "revenue": 0,
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "skills": [1],
                        "time_windows": [[0]],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "remove": ["string"],
            },
            locations=["string"],
            shipment_changes={
                "add": [
                    {
                        "delivery": {
                            "id": '"id":"Shipment Delivery 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "pickup": {
                            "id": '"id": "Shipment Pickup 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "amount": [0],
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_time_in_vehicle": 0,
                        "outsourcing_cost": 0,
                        "priority": 0,
                        "revenue": 0,
                        "skills": [0],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "modify": [
                    {
                        "delivery": {
                            "id": '"id":"Shipment Delivery 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "pickup": {
                            "id": '"id": "Shipment Pickup 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "amount": [0],
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_time_in_vehicle": 0,
                        "outsourcing_cost": 0,
                        "priority": 0,
                        "revenue": 0,
                        "skills": [0],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "remove": ["string"],
            },
            vehicle_changes={
                "add": [
                    {
                        "id": "id",
                        "location": {
                            "lat": 0,
                            "lng": 0,
                        },
                        "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                        "priority": 0,
                        "remaining_waypoints": [
                            {
                                "lat": -90,
                                "lon": -180,
                            }
                        ],
                    }
                ],
                "modify": {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                },
                "remove": ["string"],
            },
        )
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_re_optimize(self, client: NextbillionSDK) -> None:
        response = client.optimization.with_raw_response.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        optimization = response.parse()
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_re_optimize(self, client: NextbillionSDK) -> None:
        with client.optimization.with_streaming_response.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            optimization = response.parse()
            assert_matches_type(PostResponse, optimization, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncOptimization:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_compute(self, async_client: AsyncNextbillionSDK) -> None:
        optimization = await async_client.optimization.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        )
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_compute_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        optimization = await async_client.optimization.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
            approaches="`unrestricted`",
            destination="`any`",
            geometries="`polyline`",
            mode="`car`",
            roundtrip=True,
            source="`any`",
            with_geometry=True,
        )
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_compute(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.optimization.with_raw_response.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        optimization = await response.parse()
        assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_compute(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.optimization.with_streaming_response.compute(
            coordinates="coordinates=41.35544869444527,2.0747669962025292|41.37498154684205,2.103705 4530396886|41.38772862000152,2.1311887061315526",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            optimization = await response.parse()
            assert_matches_type(OptimizationComputeResponse, optimization, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_re_optimize(self, async_client: AsyncNextbillionSDK) -> None:
        optimization = await async_client.optimization.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        )
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_re_optimize_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        optimization = await async_client.optimization.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
            job_changes={
                "add": [
                    {
                        "id": '"id":"Job 1"',
                        "location_index": 0,
                        "delivery": [0],
                        "depot_ids": ["string"],
                        "description": "description",
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "outsourcing_cost": 0,
                        "pickup": [0],
                        "priority": 0,
                        "revenue": 0,
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "skills": [1],
                        "time_windows": [[0]],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "modify": [
                    {
                        "id": '"id":"Job 1"',
                        "location_index": 0,
                        "delivery": [0],
                        "depot_ids": ["string"],
                        "description": "description",
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "outsourcing_cost": 0,
                        "pickup": [0],
                        "priority": 0,
                        "revenue": 0,
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "skills": [1],
                        "time_windows": [[0]],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "remove": ["string"],
            },
            locations=["string"],
            shipment_changes={
                "add": [
                    {
                        "delivery": {
                            "id": '"id":"Shipment Delivery 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "pickup": {
                            "id": '"id": "Shipment Pickup 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "amount": [0],
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_time_in_vehicle": 0,
                        "outsourcing_cost": 0,
                        "priority": 0,
                        "revenue": 0,
                        "skills": [0],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "modify": [
                    {
                        "delivery": {
                            "id": '"id":"Shipment Delivery 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "pickup": {
                            "id": '"id": "Shipment Pickup 1"',
                            "location_index": 0,
                            "description": "description",
                            "max_visit_lateness": 0,
                            "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                            "sequence_order": 0,
                            "service": 0,
                            "setup": 0,
                            "time_windows": [[0]],
                        },
                        "amount": [0],
                        "follow_lifo_order": True,
                        "incompatible_load_types": ["string"],
                        "joint_order": 0,
                        "load_types": ["string"],
                        "max_time_in_vehicle": 0,
                        "outsourcing_cost": 0,
                        "priority": 0,
                        "revenue": 0,
                        "skills": [0],
                        "volume": {
                            "alignment": "`strict`",
                            "depth": 0,
                            "height": 0,
                            "width": 0,
                        },
                        "zones": [0],
                    }
                ],
                "remove": ["string"],
            },
            vehicle_changes={
                "add": [
                    {
                        "id": "id",
                        "location": {
                            "lat": 0,
                            "lng": 0,
                        },
                        "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                        "priority": 0,
                        "remaining_waypoints": [
                            {
                                "lat": -90,
                                "lon": -180,
                            }
                        ],
                    }
                ],
                "modify": {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                },
                "remove": ["string"],
            },
        )
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_re_optimize(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.optimization.with_raw_response.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        optimization = await response.parse()
        assert_matches_type(PostResponse, optimization, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_re_optimize(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.optimization.with_streaming_response.re_optimize(
            key="key=API_KEY",
            existing_request_id="existing_request_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            optimization = await response.parse()
            assert_matches_type(PostResponse, optimization, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    map,
    mdm,
    areas,
    batch,
    browse,
    lookup,
    geocode,
    discover,
    isochrone,
    directions,
    navigation,
    postalcode,
    revgeocode,
    autosuggest,
    autocomplete,
    restrictions,
    route_report,
    snap_to_roads,
    restrictions_items,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, NextbillionSDKError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.skynet import skynet
from .resources.fleetify import fleetify
from .resources.geofence import geofence
from .resources.multigeocode import multigeocode
from .resources.optimization import optimization
from .resources.distancematrix import distancematrix

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "NextbillionSDK",
    "AsyncNextbillionSDK",
    "Client",
    "AsyncClient",
]


class NextbillionSDK(SyncAPIClient):
    fleetify: fleetify.FleetifyResource
    skynet: skynet.SkynetResource
    geocode: geocode.GeocodeResource
    optimization: optimization.OptimizationResource
    geofence: geofence.GeofenceResource
    discover: discover.DiscoverResource
    browse: browse.BrowseResource
    mdm: mdm.MdmResource
    isochrone: isochrone.IsochroneResource
    restrictions: restrictions.RestrictionsResource
    restrictions_items: restrictions_items.RestrictionsItemsResource
    distancematrix: distancematrix.DistancematrixResource
    autocomplete: autocomplete.AutocompleteResource
    navigation: navigation.NavigationResource
    map: map.MapResource
    autosuggest: autosuggest.AutosuggestResource
    directions: directions.DirectionsResource
    batch: batch.BatchResource
    multigeocode: multigeocode.MultigeocodeResource
    revgeocode: revgeocode.RevgeocodeResource
    route_report: route_report.RouteReportResource
    snap_to_roads: snap_to_roads.SnapToRoadsResource
    postalcode: postalcode.PostalcodeResource
    areas: areas.AreasResource
    lookup: lookup.LookupResource
    with_raw_response: NextbillionSDKWithRawResponse
    with_streaming_response: NextbillionSDKWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous NextbillionSDK client instance.

        This automatically infers the `api_key` argument from the `NEXTBILLION_SDK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("NEXTBILLION_SDK_API_KEY")
        if api_key is None:
            raise NextbillionSDKError(
                "The api_key client option must be set either by passing api_key to the client or by setting the NEXTBILLION_SDK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("NEXTBILLION_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.nextbillion.io"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.fleetify = fleetify.FleetifyResource(self)
        self.skynet = skynet.SkynetResource(self)
        self.geocode = geocode.GeocodeResource(self)
        self.optimization = optimization.OptimizationResource(self)
        self.geofence = geofence.GeofenceResource(self)
        self.discover = discover.DiscoverResource(self)
        self.browse = browse.BrowseResource(self)
        self.mdm = mdm.MdmResource(self)
        self.isochrone = isochrone.IsochroneResource(self)
        self.restrictions = restrictions.RestrictionsResource(self)
        self.restrictions_items = restrictions_items.RestrictionsItemsResource(self)
        self.distancematrix = distancematrix.DistancematrixResource(self)
        self.autocomplete = autocomplete.AutocompleteResource(self)
        self.navigation = navigation.NavigationResource(self)
        self.map = map.MapResource(self)
        self.autosuggest = autosuggest.AutosuggestResource(self)
        self.directions = directions.DirectionsResource(self)
        self.batch = batch.BatchResource(self)
        self.multigeocode = multigeocode.MultigeocodeResource(self)
        self.revgeocode = revgeocode.RevgeocodeResource(self)
        self.route_report = route_report.RouteReportResource(self)
        self.snap_to_roads = snap_to_roads.SnapToRoadsResource(self)
        self.postalcode = postalcode.PostalcodeResource(self)
        self.areas = areas.AreasResource(self)
        self.lookup = lookup.LookupResource(self)
        self.with_raw_response = NextbillionSDKWithRawResponse(self)
        self.with_streaming_response = NextbillionSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @property
    @override
    def default_query(self) -> dict[str, object]:
        return {
            **super().default_query,
            "key": self.api_key,
            **self._custom_query,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncNextbillionSDK(AsyncAPIClient):
    fleetify: fleetify.AsyncFleetifyResource
    skynet: skynet.AsyncSkynetResource
    geocode: geocode.AsyncGeocodeResource
    optimization: optimization.AsyncOptimizationResource
    geofence: geofence.AsyncGeofenceResource
    discover: discover.AsyncDiscoverResource
    browse: browse.AsyncBrowseResource
    mdm: mdm.AsyncMdmResource
    isochrone: isochrone.AsyncIsochroneResource
    restrictions: restrictions.AsyncRestrictionsResource
    restrictions_items: restrictions_items.AsyncRestrictionsItemsResource
    distancematrix: distancematrix.AsyncDistancematrixResource
    autocomplete: autocomplete.AsyncAutocompleteResource
    navigation: navigation.AsyncNavigationResource
    map: map.AsyncMapResource
    autosuggest: autosuggest.AsyncAutosuggestResource
    directions: directions.AsyncDirectionsResource
    batch: batch.AsyncBatchResource
    multigeocode: multigeocode.AsyncMultigeocodeResource
    revgeocode: revgeocode.AsyncRevgeocodeResource
    route_report: route_report.AsyncRouteReportResource
    snap_to_roads: snap_to_roads.AsyncSnapToRoadsResource
    postalcode: postalcode.AsyncPostalcodeResource
    areas: areas.AsyncAreasResource
    lookup: lookup.AsyncLookupResource
    with_raw_response: AsyncNextbillionSDKWithRawResponse
    with_streaming_response: AsyncNextbillionSDKWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncNextbillionSDK client instance.

        This automatically infers the `api_key` argument from the `NEXTBILLION_SDK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("NEXTBILLION_SDK_API_KEY")
        if api_key is None:
            raise NextbillionSDKError(
                "The api_key client option must be set either by passing api_key to the client or by setting the NEXTBILLION_SDK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("NEXTBILLION_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.nextbillion.io"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.fleetify = fleetify.AsyncFleetifyResource(self)
        self.skynet = skynet.AsyncSkynetResource(self)
        self.geocode = geocode.AsyncGeocodeResource(self)
        self.optimization = optimization.AsyncOptimizationResource(self)
        self.geofence = geofence.AsyncGeofenceResource(self)
        self.discover = discover.AsyncDiscoverResource(self)
        self.browse = browse.AsyncBrowseResource(self)
        self.mdm = mdm.AsyncMdmResource(self)
        self.isochrone = isochrone.AsyncIsochroneResource(self)
        self.restrictions = restrictions.AsyncRestrictionsResource(self)
        self.restrictions_items = restrictions_items.AsyncRestrictionsItemsResource(self)
        self.distancematrix = distancematrix.AsyncDistancematrixResource(self)
        self.autocomplete = autocomplete.AsyncAutocompleteResource(self)
        self.navigation = navigation.AsyncNavigationResource(self)
        self.map = map.AsyncMapResource(self)
        self.autosuggest = autosuggest.AsyncAutosuggestResource(self)
        self.directions = directions.AsyncDirectionsResource(self)
        self.batch = batch.AsyncBatchResource(self)
        self.multigeocode = multigeocode.AsyncMultigeocodeResource(self)
        self.revgeocode = revgeocode.AsyncRevgeocodeResource(self)
        self.route_report = route_report.AsyncRouteReportResource(self)
        self.snap_to_roads = snap_to_roads.AsyncSnapToRoadsResource(self)
        self.postalcode = postalcode.AsyncPostalcodeResource(self)
        self.areas = areas.AsyncAreasResource(self)
        self.lookup = lookup.AsyncLookupResource(self)
        self.with_raw_response = AsyncNextbillionSDKWithRawResponse(self)
        self.with_streaming_response = AsyncNextbillionSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @property
    @override
    def default_query(self) -> dict[str, object]:
        return {
            **super().default_query,
            "key": self.api_key,
            **self._custom_query,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class NextbillionSDKWithRawResponse:
    def __init__(self, client: NextbillionSDK) -> None:
        self.fleetify = fleetify.FleetifyResourceWithRawResponse(client.fleetify)
        self.skynet = skynet.SkynetResourceWithRawResponse(client.skynet)
        self.geocode = geocode.GeocodeResourceWithRawResponse(client.geocode)
        self.optimization = optimization.OptimizationResourceWithRawResponse(client.optimization)
        self.geofence = geofence.GeofenceResourceWithRawResponse(client.geofence)
        self.discover = discover.DiscoverResourceWithRawResponse(client.discover)
        self.browse = browse.BrowseResourceWithRawResponse(client.browse)
        self.mdm = mdm.MdmResourceWithRawResponse(client.mdm)
        self.isochrone = isochrone.IsochroneResourceWithRawResponse(client.isochrone)
        self.restrictions = restrictions.RestrictionsResourceWithRawResponse(client.restrictions)
        self.restrictions_items = restrictions_items.RestrictionsItemsResourceWithRawResponse(client.restrictions_items)
        self.distancematrix = distancematrix.DistancematrixResourceWithRawResponse(client.distancematrix)
        self.autocomplete = autocomplete.AutocompleteResourceWithRawResponse(client.autocomplete)
        self.navigation = navigation.NavigationResourceWithRawResponse(client.navigation)
        self.map = map.MapResourceWithRawResponse(client.map)
        self.autosuggest = autosuggest.AutosuggestResourceWithRawResponse(client.autosuggest)
        self.directions = directions.DirectionsResourceWithRawResponse(client.directions)
        self.batch = batch.BatchResourceWithRawResponse(client.batch)
        self.multigeocode = multigeocode.MultigeocodeResourceWithRawResponse(client.multigeocode)
        self.revgeocode = revgeocode.RevgeocodeResourceWithRawResponse(client.revgeocode)
        self.route_report = route_report.RouteReportResourceWithRawResponse(client.route_report)
        self.snap_to_roads = snap_to_roads.SnapToRoadsResourceWithRawResponse(client.snap_to_roads)
        self.postalcode = postalcode.PostalcodeResourceWithRawResponse(client.postalcode)
        self.areas = areas.AreasResourceWithRawResponse(client.areas)
        self.lookup = lookup.LookupResourceWithRawResponse(client.lookup)


class AsyncNextbillionSDKWithRawResponse:
    def __init__(self, client: AsyncNextbillionSDK) -> None:
        self.fleetify = fleetify.AsyncFleetifyResourceWithRawResponse(client.fleetify)
        self.skynet = skynet.AsyncSkynetResourceWithRawResponse(client.skynet)
        self.geocode = geocode.AsyncGeocodeResourceWithRawResponse(client.geocode)
        self.optimization = optimization.AsyncOptimizationResourceWithRawResponse(client.optimization)
        self.geofence = geofence.AsyncGeofenceResourceWithRawResponse(client.geofence)
        self.discover = discover.AsyncDiscoverResourceWithRawResponse(client.discover)
        self.browse = browse.AsyncBrowseResourceWithRawResponse(client.browse)
        self.mdm = mdm.AsyncMdmResourceWithRawResponse(client.mdm)
        self.isochrone = isochrone.AsyncIsochroneResourceWithRawResponse(client.isochrone)
        self.restrictions = restrictions.AsyncRestrictionsResourceWithRawResponse(client.restrictions)
        self.restrictions_items = restrictions_items.AsyncRestrictionsItemsResourceWithRawResponse(
            client.restrictions_items
        )
        self.distancematrix = distancematrix.AsyncDistancematrixResourceWithRawResponse(client.distancematrix)
        self.autocomplete = autocomplete.AsyncAutocompleteResourceWithRawResponse(client.autocomplete)
        self.navigation = navigation.AsyncNavigationResourceWithRawResponse(client.navigation)
        self.map = map.AsyncMapResourceWithRawResponse(client.map)
        self.autosuggest = autosuggest.AsyncAutosuggestResourceWithRawResponse(client.autosuggest)
        self.directions = directions.AsyncDirectionsResourceWithRawResponse(client.directions)
        self.batch = batch.AsyncBatchResourceWithRawResponse(client.batch)
        self.multigeocode = multigeocode.AsyncMultigeocodeResourceWithRawResponse(client.multigeocode)
        self.revgeocode = revgeocode.AsyncRevgeocodeResourceWithRawResponse(client.revgeocode)
        self.route_report = route_report.AsyncRouteReportResourceWithRawResponse(client.route_report)
        self.snap_to_roads = snap_to_roads.AsyncSnapToRoadsResourceWithRawResponse(client.snap_to_roads)
        self.postalcode = postalcode.AsyncPostalcodeResourceWithRawResponse(client.postalcode)
        self.areas = areas.AsyncAreasResourceWithRawResponse(client.areas)
        self.lookup = lookup.AsyncLookupResourceWithRawResponse(client.lookup)


class NextbillionSDKWithStreamedResponse:
    def __init__(self, client: NextbillionSDK) -> None:
        self.fleetify = fleetify.FleetifyResourceWithStreamingResponse(client.fleetify)
        self.skynet = skynet.SkynetResourceWithStreamingResponse(client.skynet)
        self.geocode = geocode.GeocodeResourceWithStreamingResponse(client.geocode)
        self.optimization = optimization.OptimizationResourceWithStreamingResponse(client.optimization)
        self.geofence = geofence.GeofenceResourceWithStreamingResponse(client.geofence)
        self.discover = discover.DiscoverResourceWithStreamingResponse(client.discover)
        self.browse = browse.BrowseResourceWithStreamingResponse(client.browse)
        self.mdm = mdm.MdmResourceWithStreamingResponse(client.mdm)
        self.isochrone = isochrone.IsochroneResourceWithStreamingResponse(client.isochrone)
        self.restrictions = restrictions.RestrictionsResourceWithStreamingResponse(client.restrictions)
        self.restrictions_items = restrictions_items.RestrictionsItemsResourceWithStreamingResponse(
            client.restrictions_items
        )
        self.distancematrix = distancematrix.DistancematrixResourceWithStreamingResponse(client.distancematrix)
        self.autocomplete = autocomplete.AutocompleteResourceWithStreamingResponse(client.autocomplete)
        self.navigation = navigation.NavigationResourceWithStreamingResponse(client.navigation)
        self.map = map.MapResourceWithStreamingResponse(client.map)
        self.autosuggest = autosuggest.AutosuggestResourceWithStreamingResponse(client.autosuggest)
        self.directions = directions.DirectionsResourceWithStreamingResponse(client.directions)
        self.batch = batch.BatchResourceWithStreamingResponse(client.batch)
        self.multigeocode = multigeocode.MultigeocodeResourceWithStreamingResponse(client.multigeocode)
        self.revgeocode = revgeocode.RevgeocodeResourceWithStreamingResponse(client.revgeocode)
        self.route_report = route_report.RouteReportResourceWithStreamingResponse(client.route_report)
        self.snap_to_roads = snap_to_roads.SnapToRoadsResourceWithStreamingResponse(client.snap_to_roads)
        self.postalcode = postalcode.PostalcodeResourceWithStreamingResponse(client.postalcode)
        self.areas = areas.AreasResourceWithStreamingResponse(client.areas)
        self.lookup = lookup.LookupResourceWithStreamingResponse(client.lookup)


class AsyncNextbillionSDKWithStreamedResponse:
    def __init__(self, client: AsyncNextbillionSDK) -> None:
        self.fleetify = fleetify.AsyncFleetifyResourceWithStreamingResponse(client.fleetify)
        self.skynet = skynet.AsyncSkynetResourceWithStreamingResponse(client.skynet)
        self.geocode = geocode.AsyncGeocodeResourceWithStreamingResponse(client.geocode)
        self.optimization = optimization.AsyncOptimizationResourceWithStreamingResponse(client.optimization)
        self.geofence = geofence.AsyncGeofenceResourceWithStreamingResponse(client.geofence)
        self.discover = discover.AsyncDiscoverResourceWithStreamingResponse(client.discover)
        self.browse = browse.AsyncBrowseResourceWithStreamingResponse(client.browse)
        self.mdm = mdm.AsyncMdmResourceWithStreamingResponse(client.mdm)
        self.isochrone = isochrone.AsyncIsochroneResourceWithStreamingResponse(client.isochrone)
        self.restrictions = restrictions.AsyncRestrictionsResourceWithStreamingResponse(client.restrictions)
        self.restrictions_items = restrictions_items.AsyncRestrictionsItemsResourceWithStreamingResponse(
            client.restrictions_items
        )
        self.distancematrix = distancematrix.AsyncDistancematrixResourceWithStreamingResponse(client.distancematrix)
        self.autocomplete = autocomplete.AsyncAutocompleteResourceWithStreamingResponse(client.autocomplete)
        self.navigation = navigation.AsyncNavigationResourceWithStreamingResponse(client.navigation)
        self.map = map.AsyncMapResourceWithStreamingResponse(client.map)
        self.autosuggest = autosuggest.AsyncAutosuggestResourceWithStreamingResponse(client.autosuggest)
        self.directions = directions.AsyncDirectionsResourceWithStreamingResponse(client.directions)
        self.batch = batch.AsyncBatchResourceWithStreamingResponse(client.batch)
        self.multigeocode = multigeocode.AsyncMultigeocodeResourceWithStreamingResponse(client.multigeocode)
        self.revgeocode = revgeocode.AsyncRevgeocodeResourceWithStreamingResponse(client.revgeocode)
        self.route_report = route_report.AsyncRouteReportResourceWithStreamingResponse(client.route_report)
        self.snap_to_roads = snap_to_roads.AsyncSnapToRoadsResourceWithStreamingResponse(client.snap_to_roads)
        self.postalcode = postalcode.AsyncPostalcodeResourceWithStreamingResponse(client.postalcode)
        self.areas = areas.AsyncAreasResourceWithStreamingResponse(client.areas)
        self.lookup = lookup.AsyncLookupResourceWithStreamingResponse(client.lookup)


Client = NextbillionSDK

AsyncClient = AsyncNextbillionSDK



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .json_retrieve_params import JsonRetrieveParams as JsonRetrieveParams
from .json_retrieve_response import JsonRetrieveResponse as JsonRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .routes.route_steps_request_param import RouteStepsRequestParam

__all__ = ["RouteCreateParams", "Routing"]


class RouteCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    driver_email: Required[str]
    """Specify the e-mail address of the driver who should receive the route.

    The e-mail address must be registered in
    [NextBillion.ai Cloud Console](https://console.nextbillion.ai/).
    """

    steps: Required[Iterable[RouteStepsRequestParam]]
    """
    An array of objects to collect the details about the intermediate steps in the
    route to be dispatched. Each object corresponds to a single step. The array must
    begin with a start-type step and end with an end-type step, to form a valid
    route.
    """

    distance: int
    """
    Specify the total distance, in meters, for an informative display in Driver's
    app. The distance specified here has no effect on the actual route that the
    service generates.
    """

    document_template_id: str
    """
    Specify the ID of the document template that should be used to collect proof of
    completion for all steps in the route. In order to complete each route step, the
    driver will need to submit a form generated by the rules defined in the given
    document template. Use the
    [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
    create, read and manage document templates.

    Please note that the document template ID assigned to a route does not apply to
    following step types - `start`, `end`, `break`, `layover`.
    """

    ro_request_id: str
    """Specify the Route Optimization request ID.

    When this ID is provided, all other fields will be ignored (including the
    required fields) and the route optimization result will be used to form the
    routes and corresponding steps.

    Please note that:

    - The driver's email ID must be provided in input `vehicle.metadata` as
      `user_email` such that the route optimization result must contain a valid
      driver email, step's arrival time, etc., to make a successful dispatch.
    - Document Template for collecting proof of delivery or completion can not be
      specified when using this field to dispatch a route.
    - In case of an error at any part among the routes, the API will immediately
      return the error with the index of the specific route or route step.
    - On a successful dispatch, the API returns the last route, if there are many,
      in the response payload.
    """

    routing: Routing
    """
    The `routing` object allows defining the routing characteristics that should be
    used to generate a route when the Driver uses the in-app navigation. Only `car`
    mode is supported currently.
    """


class Routing(TypedDict, total=False):
    approaches: Literal["`unrestricted`", "`curb`"]
    """
    Specify the side of the road from which the route should approach the step
    location. When set to `unrestricted` a route can arrive at the step location
    from either side of the road and when set to `curb` the route will arrive at the
    step location only from the driving side of the region. Use a semi-colon `;` to
    specify approach configurations for multiple steps.
    """

    avoid: Literal[
        "`toll`",
        "`highway`",
        "`ferry`",
        "`sharp_turn`",
        "`uturn`",
        "`left_turn`",
        "`right_turn`",
        "`service_road`",
        "`none`",
    ]
    """
    Setting this will ensure the generated route avoids the object(s) specified in
    the input. Multiple values should be separated by a pipe (|). If `none` is
    provided along with other values, an error is returned as a valid route is not
    feasible.
    """

    hazmat_type: Literal["`general`", "`circumstantial`", "`explosive`", "`harmful_to_water`"]
    """
    Specify the type of hazardous material being carried and the dispatch service
    will avoid roads which are not suitable for the type of goods specified.
    Multiple values can be separated using a pipe operator `|` .

    Please note that this parameter is effective only when `mode=truck`.
    """

    mode: Literal["`car`"]
    """Specify the driving mode that the service should use to determine a route"""

    truck_axle_load: int
    """
    Specify the total load per axle (including the weight of trailers and shipped
    goods) of the truck, in tonnes. When specified, the dispatched route uses only
    those roads which can be used by a truck to carry the specified load per axle.

    Please note this parameter is effective only when `mode=truck`.
    """

    truck_size: str
    """
    Specify the dimensions of a truck, in centimeters (cm), in the format of
    <height, width, length>. When specified, the dispatched route uses only those
    roads which allow trucks with specified dimensions.

    Please note this parameter is effective only when `mode=truck`. Also, the
    maximum dimensions that can be specified are as follows:

    Height = 1000 cm
    Width = 5000 cm
    Length = 5000 cm
    """

    truck_weight: int
    """
    Specify the weight of the truck, including trailers and shipped goods, in
    kilograms (kg). When specified, the dispatched route uses only those roads which
    allow trucks with specified weight.

    Please note this parameter is effective only when `mode=truck`. Also, the
    maximum weight that can be specified for a truck is 100,000 kgs.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RouteReportCreateParams"]


class RouteReportCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    original_shape: Required[str]
    """Takes a route geometry as input and returns the route details.

    Accepts `polyline` and `polyline6` encoded geometry as input.

    **Note**: Route geometries generated from sources other than
    [NextBillion.ai](http://NextBillion.ai) services, are not supported in this
    version.
    """

    original_shape_type: Required[Literal["`polyline`", "`polyline6`"]]
    """Specify the encoding type of route geometry provided in `original_shape` input.

    Please note that an error is returned when this parameter is not specified while
    an input is added to `original_shape` parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "SnapToRoadSnapResponse",
    "Geojson",
    "GeojsonGeometry",
    "RoadInfo",
    "RoadInfoMaxSpeed",
    "SnappedPoint",
    "SnappedPointLocation",
]


class GeojsonGeometry(BaseModel):
    coordinates: Optional[List[float]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    snapped path geometry.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry"""


class Geojson(BaseModel):
    geometry: Optional[GeojsonGeometry] = None
    """An object with details of the geoJSON geometry of the snapped path."""

    properties: Optional[str] = None
    """Properties associated with the geoJSON shape of the snapped path."""

    type: Optional[str] = None
    """Type of the GeoJSON object."""


class RoadInfoMaxSpeed(BaseModel):
    length: Optional[int] = None
    """
    `length` refers to a sequence of 'n' consecutive vertices in the route geometry
    starting from the `offset`, forming a continuous section of route where the
    maximum speed is the same and is indicated in `value`.
    """

    offset: Optional[int] = None
    """
    `offset` is the index value of the vertex of route geometry, which is the
    starting point of the segment.
    """

    value: Optional[float] = None
    """`value` denotes the maximum speed of this segment, in kilometers per hour.

    - A value of "-1" indicates that the speed is unlimited for this road segment.
    - A value of "0" indicates that there is no information about the maximum speed
      for this road segment.
    """


class RoadInfo(BaseModel):
    max_speed: Optional[List[RoadInfoMaxSpeed]] = None
    """
    An array of objects containing maximum speed, in kilometers per hour, for each
    segment of the route. Each object represents one road segment.
    """


class SnappedPointLocation(BaseModel):
    latitude: float
    """Latitude of the snapped point."""

    longitude: float
    """Longitude of the snapped point."""


class SnappedPoint(BaseModel):
    bearing: float
    """
    The bearing, calculated as the angle from true north in clockwise direction, of
    the route leading to the next snapped point from the current `snapped_point`, in
    radians. In case of the last `snapped_point` of the route, the bearing indicates
    the direction of the route to the previous `snapped_location`.
    """

    distance: float
    """The distance of the snapped point from the original input coordinate in meters."""

    location: SnappedPointLocation
    """The latitude and longitude coordinates of the snapped point."""

    name: str
    """The name of the street or road that the input coordinate snapped to."""

    original_index: int = FieldInfo(alias="originalIndex")
    """
    The index of the input `path` coordinate point to which this snapped point
    corresponds to.
    """


class SnapToRoadSnapResponse(BaseModel):
    distance: Optional[int] = None
    """The total distance of the snapped path in meters."""

    geojson: Optional[Geojson] = None
    """A GeoJSON object with details of the snapped path.

    This object is returned when the `geometry` field is set to `geojson` in the
    input request, otherwise it is not present in the response. The contents of this
    object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    geometry: Optional[List[str]] = None
    """
    An array of strings containing the encoded geometries of snapped paths in
    `polyline` or `polyline6` format.
    """

    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    road_info: Optional[RoadInfo] = None
    """
    An object containing the maximum speed information for each road segment present
    in the route.
    """

    snapped_points: Optional[List[SnappedPoint]] = FieldInfo(alias="snappedPoints", default=None)
    """An array of objects.

    Each object provides the details of a `path` coordinate point snapped to the
    nearest road.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the [API Errors Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.skynet.asset import location_list_params, location_get_last_params
from ....types.skynet.asset.location_list_response import LocationListResponse
from ....types.skynet.asset.location_get_last_response import LocationGetLastResponse

__all__ = ["LocationResource", "AsyncLocationResource"]


class LocationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LocationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LocationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LocationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return LocationResourceWithStreamingResponse(self)

    def list(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        correction: str | NotGiven = NOT_GIVEN,
        end_time: int | NotGiven = NOT_GIVEN,
        geometry_type: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        start_time: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocationListResponse:
        """
        Track locations of an asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          correction: Describe the geometry characteristics through a `,` separated list of
              properties.

              Setting `mapmatch` to 1 returns the geometry of the tracked points, snapped to
              the nearest road.

              Setting `interpolate` to 1 smoothens the snapped geometry by adding more points,
              as needed. Please note, `mapmatch` should be set to 1 for `interpolate` to be
              effective.

              `mode` is used to set the transport mode for which the snapped route will be
              determined. Allowed values for `mode` are `car` and `truck`.

          end_time: Time until which the tracked locations of the `asset` need to be retrieved.

          geometry_type: Set the geometry format to encode the path linking the tracked locations of the
              `asset`.

              Please note that `geometry_type` is effective only when `mapmatch` property of
              `correction` parameter is set to 1.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          start_time: Time after which the tracked locations of the `asset` need to be retrieved.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/asset/{id}/location/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "correction": correction,
                        "end_time": end_time,
                        "geometry_type": geometry_type,
                        "pn": pn,
                        "ps": ps,
                        "start_time": start_time,
                    },
                    location_list_params.LocationListParams,
                ),
            ),
            cast_to=LocationListResponse,
        )

    def get_last(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocationGetLastResponse:
        """
        Track the last location of an asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/asset/{id}/location/last",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    location_get_last_params.LocationGetLastParams,
                ),
            ),
            cast_to=LocationGetLastResponse,
        )


class AsyncLocationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLocationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLocationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLocationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncLocationResourceWithStreamingResponse(self)

    async def list(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        correction: str | NotGiven = NOT_GIVEN,
        end_time: int | NotGiven = NOT_GIVEN,
        geometry_type: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        start_time: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocationListResponse:
        """
        Track locations of an asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          correction: Describe the geometry characteristics through a `,` separated list of
              properties.

              Setting `mapmatch` to 1 returns the geometry of the tracked points, snapped to
              the nearest road.

              Setting `interpolate` to 1 smoothens the snapped geometry by adding more points,
              as needed. Please note, `mapmatch` should be set to 1 for `interpolate` to be
              effective.

              `mode` is used to set the transport mode for which the snapped route will be
              determined. Allowed values for `mode` are `car` and `truck`.

          end_time: Time until which the tracked locations of the `asset` need to be retrieved.

          geometry_type: Set the geometry format to encode the path linking the tracked locations of the
              `asset`.

              Please note that `geometry_type` is effective only when `mapmatch` property of
              `correction` parameter is set to 1.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          start_time: Time after which the tracked locations of the `asset` need to be retrieved.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/asset/{id}/location/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "correction": correction,
                        "end_time": end_time,
                        "geometry_type": geometry_type,
                        "pn": pn,
                        "ps": ps,
                        "start_time": start_time,
                    },
                    location_list_params.LocationListParams,
                ),
            ),
            cast_to=LocationListResponse,
        )

    async def get_last(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LocationGetLastResponse:
        """
        Track the last location of an asset

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/asset/{id}/location/last",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    location_get_last_params.LocationGetLastParams,
                ),
            ),
            cast_to=LocationGetLastResponse,
        )


class LocationResourceWithRawResponse:
    def __init__(self, location: LocationResource) -> None:
        self._location = location

        self.list = to_raw_response_wrapper(
            location.list,
        )
        self.get_last = to_raw_response_wrapper(
            location.get_last,
        )


class AsyncLocationResourceWithRawResponse:
    def __init__(self, location: AsyncLocationResource) -> None:
        self._location = location

        self.list = async_to_raw_response_wrapper(
            location.list,
        )
        self.get_last = async_to_raw_response_wrapper(
            location.get_last,
        )


class LocationResourceWithStreamingResponse:
    def __init__(self, location: LocationResource) -> None:
        self._location = location

        self.list = to_streamed_response_wrapper(
            location.list,
        )
        self.get_last = to_streamed_response_wrapper(
            location.get_last,
        )


class AsyncLocationResourceWithStreamingResponse:
    def __init__(self, location: AsyncLocationResource) -> None:
        self._location = location

        self.list = async_to_streamed_response_wrapper(
            location.list,
        )
        self.get_last = async_to_streamed_response_wrapper(
            location.get_last,
        )
